@@ -3,7 +3,7 @@ from .endgame import Endgame
 from .errors import *
 from .fortress import Fortress
 from .htb import HTBClient, HTBObject
-from .machine import Machine
+from .machine import Machine,MachineInstance
 from .search import Search
 from .solve import *
 from .team import Team
