@@ -17,7 +17,7 @@ from .errors import AuthenticationException, NotFoundException, \
 if TYPE_CHECKING:
     from .user import User
     from .search import Search
-    from .machine import Machine
+    from .machine import Machine, MachineInstance
     from .challenge import Challenge
     from .endgame import Endgame
     from .fortress import Fortress
@@ -57,8 +57,9 @@ class HTBClient:
     """
     # noinspection PyUnresolvedReferences
     _user: Optional["User"] = None
-    _access_token: str
-    _refresh_token: str
+    _access_token: Optional[str]
+    _refresh_token: Optional[str]
+    _app_token: Optional[str]
     _api_base: str
     challenge_cooldown: int = 0
 
@@ -98,9 +99,14 @@ class HTBClient:
         if authorized:
             # Don't use authorization if the API base URL isn't the real one -
             # i.e. we're running a test
-            if jwt_expired(self._access_token):
-                self._refresh_access_token()
-            headers['Authorization'] = "Bearer " + self._access_token
+            if self._app_token is not None:
+                headers['Authorization'] = "Bearer " + self._app_token
+            elif self._access_token is not None and self._refresh_token is not None:
+                if jwt_expired(self._access_token):
+                    self._refresh_access_token()
+                headers['Authorization'] = "Bearer " + self._access_token
+            else:
+                raise AuthenticationException("No authentication tokens available")
         while True:
             if not json_data and not data:
                 if post:
@@ -124,7 +130,8 @@ class HTBClient:
             return r.json()
 
     def __init__(self, email: Optional[str] = None, password: Optional[str] = None, otp: Optional[str | int] = None,
-                 cache: Optional[str] = None, api_base: str = API_BASE, remember: Optional[bool] = False):
+                 cache: Optional[str] = None, api_base: str = API_BASE, remember: Optional[bool] = False,
+                 app_token: Optional[str] = None):
         """
         Authenticates to the API.
 
@@ -138,16 +145,17 @@ class HTBClient:
             otp: The current OTP of the user, if 2FA is enabled
             cache: The path to load/store access tokens from/to
             remember: Whether to create a long-lasting 'remember me' token
+            app_token: Authenticate using a provided App Token
         """
         self._api_base = api_base
         if cache is not None:
             if self.load_from_cache(cache) is False:
-                self.do_login(email, password, otp, remember)
+                self.do_login(email, password, otp, remember, app_token)
                 self.dump_to_cache(cache)
             # Make sure we dump our current tokens out when we exit
             atexit.register(self.dump_to_cache, cache)
         else:
-            self.do_login(email, password, otp, remember)
+            self.do_login(email, password, otp, remember, app_token)
 
     def load_from_cache(self, cache: str) -> bool:
         """
@@ -160,14 +168,16 @@ class HTBClient:
             return False
         with open(cache, 'r') as f:
             data = json.load(f)
-        self._access_token = data['access_token']
-        self._refresh_token = data['refresh_token']
-        if jwt_expired(self._access_token):
-            try:
-                self._refresh_access_token()
-            # Our refresh token is also invalid, we must log in again
-            except AuthenticationException:
-                return False
+        self._access_token = data.get('access_token')
+        self._refresh_token = data.get('refresh_token')
+        self._app_token = data.get('app_token')
+        if self._access_token is not None:
+            if jwt_expired(self._access_token):
+                try:
+                    self._refresh_access_token()
+                # Our refresh token is also invalid, we must log in again
+                except AuthenticationException:
+                    return False
         return True
 
     def dump_to_cache(self, cache):
@@ -179,41 +189,47 @@ class HTBClient:
         with open(cache, 'w') as f:
             json.dump({
                 "access_token": self._access_token,
-                "refresh_token": self._refresh_token
+                "refresh_token": self._refresh_token,
+                "app_token": self._app_token
             }, f)
 
     def do_login(self, email: Optional[str] = None, password: Optional[str] = None, otp: Optional[str | int] = None,
-                 remember: Optional[bool] = False):
+                 remember: Optional[bool] = False, app_token: Optional[str] = None):
         """
         Authenticates against the API. If credentials are not provided, they will be prompted for.
         """
-        if email is None:
-            email = input("Email: ")
-        if password is None:
-            password = getpass.getpass()
 
-        data = cast(dict, self.do_request("login", json_data={
-            "email": email, "password": password, "remember": remember
-        }, authorized=False))
-        msg = data['message']
+        self._app_token = app_token
+        if app_token is not None:
+            self._access_token = self._refresh_token = None
+        else:
+            if email is None:
+                email = input("Email: ")
+            if password is None:
+                password = getpass.getpass()
 
-        self._access_token = msg.get('access_token')
-        if self._access_token is None:
-            raise ApiError(f"Failed to get access token: {msg}")
-        self._refresh_token = msg.get('refresh_token')
-        if self._refresh_token is None:
-            raise ApiError(f"Failed to get refresh token: {msg}")
-        if data['message']['is2FAEnabled'] is True:
-            if otp is None:
-                otp = input("OTP: ")
-            if type(otp) == int:
-                # Optimistically try and create a string
-                otp = f"{otp:06d}"
-            resp = cast(dict, self.do_request("2fa/login", json_data={
-                "one_time_password": otp
-            }))
-            if "correct" not in resp['message']:
-                raise IncorrectOTPException
+            data = cast(dict, self.do_request("login", json_data={
+                "email": email, "password": password, "remember": remember
+            }, authorized=False))
+            msg = data['message']
+
+            self._access_token = msg.get('access_token')
+            if self._access_token is None:
+                raise ApiError(f"Failed to get access token: {msg}")
+            self._refresh_token = msg.get('refresh_token')
+            if self._refresh_token is None:
+                raise ApiError(f"Failed to get refresh token: {msg}")
+            if data['message']['is2FAEnabled'] is True:
+                if otp is None:
+                    otp = input("OTP: ")
+                if type(otp) == int:
+                    # Optimistically try and create a string
+                    otp = f"{otp:06d}"
+                resp = cast(dict, self.do_request("2fa/login", json_data={
+                    "one_time_password": otp
+                }))
+                if "correct" not in resp['message']:
+                    raise IncorrectOTPException
 
     # noinspection PyUnresolvedReferences
     def search(self, search_term: str) -> "Search":
@@ -242,6 +258,43 @@ class HTBClient:
         data = cast(dict, self.do_request(f"machine/profile/{machine_id}"))['info']
         return Machine(data, self)
 
+    # noinspection PyUnresolvedReferences
+    def get_todo_machines(self, limit: int = None) -> List[int]:
+        """
+
+        Retrieve a list of `Machine` ID's from the API based on the users todo list
+
+        Args:
+            limit: The maximum number to fetch
+
+        Returns: A list of `Machine` ID's
+
+        """
+        data = cast(dict, self.do_request("home/user/todo"))['data']['machines'][:limit]
+        return [m['id'] for m in data]
+
+
+    # noinspection PyUnresolvedReferences
+    def get_active_machine(self, release_arena: bool=False) -> Optional["MachineInstance"]:
+        """
+
+        Retrieve `Machine` currently assigned to user
+
+        Returns: The `Machine` currently assigned (or active) to user
+
+        """
+        from .machine import Machine, MachineInstance
+        if release_arena:
+            info = cast(dict, self.do_request(f"release_arena/active"))['info']
+        else:
+            info = cast(dict, self.do_request(f"machine/active"))['info']
+        if info:
+            box = self.get_machine(info['id'])
+            server = box._client.get_current_vpn_server(release_arena)
+            return MachineInstance(box.ip, server, box, box._client)
+        return None
+        
+
     # noinspection PyUnresolvedReferences
     def get_machines(self, limit: int = None, retired: bool = False) -> List["Machine"]:
         """
@@ -260,7 +313,10 @@ class HTBClient:
             data = cast(dict, self.do_request("machine/list"))['info'][:limit]
         else:
             data = cast(dict, self.do_request("machine/list/retired"))['info'][:limit]
-        return [Machine(m, self, summary=True) for m in data]
+        machines = [Machine(m, self, summary=True) for m in data]
+        for machine in machines:
+            machine.retired = retired
+        return machines
 
     # noinspection PyUnresolvedReferences
     def get_challenge(self, challenge_id: int | str) -> "Challenge":
@@ -434,7 +490,7 @@ class HTBClient:
         return Leaderboard(data, self, University)
 
     # noinspection PyUnresolvedReferences
-    def get_current_vpn_server(self, release_arena=False) -> "VPNServer":
+    def get_current_vpn_server(self, release_arena=False) -> VPNServer:
         """
         Returns: The currently assigned `VPNServer`
 
@@ -442,10 +498,11 @@ class HTBClient:
             release_arena: Get the current release arena VPN server
         """
         from .vpn import VPNServer
-        connections = cast(dict, self.do_request('connections'))['data']
         if release_arena:
-            data = connections['release_arena']['assigned_server']
+            connections = cast(dict, self.do_request('connections/servers?product=release_arena'))['data']
+            data = connections['assigned']
         else:
+            connections = cast(dict, self.do_request('connections'))['data']
             data = connections['lab']['assigned_server']
 
         return VPNServer(data, self)
